--
-- Rekordia
-- Specialization for Rekordia
--
-- @author Felix "Outlaw" Sorge
-- @date  20/05/09
-- @web www.ls-modsource.de - www.ls-mods.de
--
-- Copyright (C) Outlaw, All Rights Reserved.
--

Rekordia = {};

function Rekordia.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function Rekordia:load(xmlFile)

    assert(self.setIsTurnedOn == nil, "Sprayer needs to be the first specialization which implements setIsTurnedOn");
    self.setIsTurnedOn = Sprayer.setIsTurnedOn;
    self.getIsTurnedOnAllowed = Sprayer.getIsTurnedOnAllowed;

    self.setIsWaterTankFilling = Utils.overwrittenFunction(self.setIsWaterTankFilling, Rekordia.setIsWaterTankFilling);
	self.fertilizerInRange = SpecializationUtil.callSpecializationsFunction("fertilizerInRange");
	self.manureTanker = true;	
	self.literPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.literPerSecond#value"), 1);
	self.manureTriggerCallback = Rekordia.manureTriggerCallback;
	self.trigger = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.Trigger#index"));	
	addTrigger(self.trigger, "manureTriggerCallback", self);
	self.deleteListenerId = addDeleteListener(self.trigger, "delete", self);
	
	self.chargeTypes = {};
	self.fillTypes[FruitUtil.FRUITTYPE_UNKNOWN] = true;
	local chargeTypes = getXMLString(xmlFile, "vehicle.chargeTypes#chargeTypes");
    if chargeTypes ~= nil then
		self.chargefillNode = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.chargeTypes#fillNode"));
        local types = Utils.splitString(" ", chargeTypes);
        for k,v in pairs(types) do
            local desc = FruitUtil.fruitTypes[v];
            if desc ~= nil then
                self.chargeTypes[desc.index] = true;
            end;
        end;
    end;
	
    assert(self.setIsSprayerFilling == nil, "Sprayer needs to be the first specialization which implements setIsSprayerFilling");
    self.setIsSprayerFilling = Rekordia.setIsSprayerFilling;
    self.addSprayerFillTrigger = Rekordia.addSprayerFillTrigger;
    self.removeSprayerFillTrigger = Rekordia.removeSprayerFillTrigger;

    self.fillLitersPerSecond = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillLitersPerSecond"), 500);
    self.isSprayerFilling = false;

	local workSound = getXMLString(xmlFile, "vehicle.workSound#file");
    if workSound ~= nil and workSound ~= "" then
        workSound = Utils.getFilename(workSound, self.baseDirectory); 
        self.workSound = createSample("workSound");
        self.workSoundEnabled = false;
        loadSample(self.workSound, workSound, false);
    end;

	
	self.keyType = InputBinding.getKeyNamesOfDigitalAction;
	self.keyName = "Taste";	
	self.fill = false;
	self.fillSpeed = 1.00;
	self.trailerInTrigger = nil;
    self.sprayerFillTriggers = {};
    self.sprayerFillActivatable = SprayerFillActivatable:new(self);
    self.speedViolationMaxTime = 1000;
    self.speedViolationTimer = self.speedViolationMaxTime;	
	
end;


function Rekordia:update(dt)

    --[[if self.isClient then
        if self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
                self:setIsTurnedOn(not self.isTurnedOn);
            end;
        end;
    end;]]

end;

function Rekordia:updateTick(dt)
							
	local manureFill = self.fill;
	if self.trailerInTrigger ~= nil and self.fillLevel > 0 then
		if self:getIsActiveForInput() then		
			if self.trailerInTrigger.fillLevel < self.trailerInTrigger.capacity then
				if InputBinding.hasEvent(InputBinding.pesticideTrailer_ON)  then
					self.fill = not self.fill; 
				end;
				--[[if self.fill then
					g_currentMission:addExtraPrintText(g_i18n:getText("pesticideTrailer_off").." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.pesticideTrailer_on)); 
				else
					g_currentMission:addExtraPrintText(g_i18n:getText("pesticideTrailer_on" ).." "..InputBinding.getKeyNamesOfDigitalAction(InputBinding.pesticideTrailer_off));
				end;]]
			end;
		end;
	end;
	if self.trailerInTrigger ~= nil and self.fillLevel > 0 then	
		if g_currentMission.controlledVehicle ~= nil then
			if self.trailerInTrigger.attacherVehicle == g_currentMission.controlledVehicle or self.trailerInTrigger == g_currentMission.controlledVehicle then
				--[[if g_gui.currentGui == nil then
				end;]]
			end;
		end;
		local trailer = self.trailerInTrigger;
		if trailer.fillLevel < trailer.capacity then
			if g_server ~= nil then
				if self.fill then							
					local deltaFillLevel = dt*self.fillSpeed;
					trailer:setFillLevel(trailer.fillLevel+deltaFillLevel, self.currentFillType);
					self:setFillLevel(self.fillLevel-deltaFillLevel, self.currentFillType);
				end;
			end;
		else
			self.fill = true;
		end;
	else
		self.fill = true;
	end;
	if manureFill ~= self.fill then
		RekordiaEvent.sendEvent(self, self.fill);
	end;
	
   	self.inRange = false; 
    local nearestDistance = 10.0;
	local px, py, pz = getWorldTranslation(self.chargefillNode);
	for i=1, table.getn(g_currentMission.attachables) do
		local vx, vy, vz = getWorldTranslation(g_currentMission.attachables[i].rootNode);
		local distance = Utils.vector3Length(px-vx, py-vy, pz-vz);

		if distance < nearestDistance then
			local attachable = g_currentMission.attachables[i];	
			if fillType == self.currentFillType and attachable.rootNode ~= self.rootNode and attachable.setFillLevel ~= nil then
				            
				if attachable.fillLevel == attachable.capacity  or self.filllevel == 0 then
					self.turnOn = false; 
					if self.workSoundEnabled then
						stopSample(self.workSound);
						self.workSoundEnabled = false;
					end;
				elseif attachable.fillLevel ~= attachable.capacity  and self.fillLevel ~= 0  then
					if self.turnOn then
						self.inRange = true; 
						local deltaLevel = self.literPerSecond*dt/1000.0;
						deltaLevel = math.min(deltaLevel, attachable.capacity - attachable.fillLevel);
						self.fillLevel = self.fillLevel-deltaLevel;
						self:setFillLevel(self.fillLevel, self.currentFillType);
						attachable:setFillLevel(attachable.fillLevel + deltaLevel, self.currentFillType);	
					end; 							
				end; 
			end;
		end;
    end;
	
    if self.isSprayerFilling and self.isServer then
        local disableFilling = false;
        if self:allowFillType(self.sprayerFillingFillType, false) then
            local oldFillLevel = self.fillLevel;

            local delta = self.fillLitersPerSecond*dt*0.001;

            local silo = g_currentMission:getSiloAmount(self.sprayerFillingFillType);
            if self.sprayerFillingIsSiloTrigger then
                if silo <= 0 then
                    disableFilling = true;
                end;
                delta = math.min(delta, silo);
            end;

            self:setFillLevel(self.fillLevel + delta, self.sprayerFillingFillType, true);
            local delta = self.fillLevel - oldFillLevel;
            if delta > 0 then
                if self.sprayerFillingIsSiloTrigger then
                    g_currentMission:setSiloAmount(self.sprayerFillingFillType, silo - delta);
                else
                    local sprayType = Rekordia.fillTypeToSprayType[self.sprayerFillingFillType];
                    if sprayType ~= nil then
                        local sprayTypeDesc = Rekordia.sprayTypeIndexToDesc[sprayType]

                        local price = delta*sprayTypeDesc.pricePerLiter;
                        g_currentMission.missionStats.expensesTotal = g_currentMission.missionStats.expensesTotal + price;
                        g_currentMission.missionStats.expensesSession = g_currentMission.missionStats.expensesSession + price;

                        g_currentMission:addSharedMoney(-price);
                    end;
                end;
            elseif self.fillLevel == self.capacity then
                disableFilling = true;
                self:setIsTurnedOn(true);
            end;
        else
            disableFilling = true;
        end;
        if disableFilling then
            self:setIsSprayerFilling(false);
        end;
    end;	
	
	if not self.inRange and self.turnOn and self.workSoundEnabled then
		if self.workSoundEnabled then
			self.turnOn = not self.turnOn; 
			stopSample(self.workSound);
			self.workSoundEnabled = false;
		end;
	end;
end;


function Rekordia:fertilizerInRange(dt)
end;


function Rekordia:delete()
	removeTrigger(self.trigger);
    removeDeleteListener(self.trigger, self.deleteListenerId);
    g_currentMission:removeActivatableObject(self.sprayerFillActivatable);

end;

function Rekordia:readStream(streamId, connection)
	self.fill = streamReadBool(streamId);
    local isSprayerFilling = streamReadBool(streamId);
    self:setIsTurnedOn(turnedOn, true);
    self:setIsSprayerFilling(isSprayerFilling, true);
end;

function Rekordia:writeStream(streamId, connection)
	streamWriteBool(streamId, self.fill);
    streamWriteBool(streamId, self.isTurnedOn);
    streamWriteBool(streamId, self.isSprayerFilling);
end;


function Rekordia:mouseEvent(posX, posY, isDown, isUp, button)
end;


function Rekordia:keyEvent(unicode, sym, modifier, isDown)

end;


function Rekordia:draw()

    --[[if self.isClient then
        if self.fillLevel <= 0 and self.capacity ~= 0 then
            g_currentMission:addExtraPrintText(g_i18n:getText("FirstFillTheTool"));
        end;
        if self:getIsActiveForInput() then
            if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA) then
                self:setIsTurnedOn(not self.isTurnedOn);
            end;
        end;
        if self.isTurnedOn then
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_off_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
        else
            g_currentMission:addHelpButtonText(string.format(g_i18n:getText("turn_on_OBJECT"), self.typeDesc), InputBinding.IMPLEMENT_EXTRA);
        end;
	end;]]
	
end;

function Rekordia:setIsWaterTankFilling(superFunc, isFilling, noEventSend)
    -- Do your logic here
    print("Cur filling: "..tostring(self.isWaterTankFilling));
    print("New filling: "..tostring(isFilling));

    -- also execute the original code
    superFunc(self, isFilling, noEventSend);
end;

function Rekordia:onAttach()
end; 	


function Rekordia:onDetach()
end;

function Rekordia:manureTriggerCallback(triggerId, otherId, onEnter, onLeave, onStay, otherShapeId)
    if onEnter then
		if otherId ~= nil then
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				if trailer.fillTypes ~= nil and trailer.setFillLevel ~= nil and trailer.fillLevel ~= nil and trailer.rootNode ~= self.rootNode and trailer:allowFillType(self.currentFillType, true) then
					self.trailerInTrigger = trailer;
				end;
			end;
		end;		
	elseif onLeave then
		if otherId ~= nil then		
			local trailer = g_currentMission.objectToTrailer[otherId];
			if trailer ~= nil then
				if self.trailerInTrigger == trailer then
					self.trailerInTrigger = nil;
				end;
			end;
		end;
	end;
end;

function Rekordia:getIsTurnedOnAllowed(isTurnedOn)
    if not isTurnedOn or self.fillLevel > 0 or self.capacity == 0 then
        return true;
    end;
end;

function Rekordia:setIsTurnedOn(isTurnedOn, noEventSend)
    if isTurnedOn ~= self.isTurnedOn then
        if self:getIsTurnedOnAllowed(isTurnedOn) then
            SetTurnedOnEvent.sendEvent(self, isTurnedOn, noEventSend)

            self.isTurnedOn = isTurnedOn;
            if self.isClient then
                if not self.isTurnedOn and self.spraySoundEnabled then
                    stopSample(self.spraySound);
                    self.spraySoundEnabled = false;
                end;
            end;
        end;
    end;
end;

function Rekordia:setIsSprayerFilling(isFilling, fillType, isSiloTrigger, noEventSend)
    SprayerSetIsFillingEvent.sendEvent(self, isFilling, fillType, isSiloTrigger, noEventSend)
    if self.isSprayerFilling ~= isFilling then
        self.isSprayerFilling = isFilling;
        self.sprayerFillingFillType = fillType;
        self.sprayerFillingIsSiloTrigger = isSiloTrigger;
    end;
end;

function Rekordia:addSprayerFillTrigger(trigger)
    if table.getn(self.sprayerFillTriggers) == 0 then
        g_currentMission:addActivatableObject(self.sprayerFillActivatable);
    end;
    table.insert(self.sprayerFillTriggers, trigger);
end;

function Rekordia:removeSprayerFillTrigger(trigger)
    for i=1, table.getn(self.sprayerFillTriggers) do
        if self.sprayerFillTriggers[i] == trigger then
            table.remove(self.sprayerFillTriggers, i);
            break;
        end;
    end;
    if table.getn(self.sprayerFillTriggers) == 0 then
        if self.isServer then
            self:setIsSprayerFilling(false);
        end;
        g_currentMission:removeActivatableObject(self.sprayerFillActivatable);
    end;
end;

function Rekordia:getIsTurnedOnAllowed(isTurnedOn)
    if not isTurnedOn or self.fillLevel > 0 or self.capacity == 0 then
        return true;
    end;
end;
------------------------------------------------------------ EVENT ------------------------------------------------------------
RekordiaEvent = {};
RekordiaEvent_mt = Class(RekordiaEvent, Event);

InitEventClass(RekordiaEvent, "RekordiaEvent");

function RekordiaEvent:emptyNew()
    local self = Event:new(RekordiaEvent_mt);
    self.className="RekordiaEvent";
    return self;
end;

function RekordiaEvent:new(object, data)
    local self = RekordiaEvent:emptyNew()
    self.object = object;
	self.data = data;
    return self;
end;

function RekordiaEvent:readStream(streamId, connection)
    local id = streamReadInt32(streamId);
	self.object = networkGetObject(id);
	self.data = streamReadBool(streamId);
	self.object.fill = self.data;
	if not connection:getIsServer() then
        g_server:broadcastEvent(RekordiaEvent:new(self.object, self.data), nil, connection, self.object);
    end;
end;

function RekordiaEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.data);
end;

function RekordiaEvent.sendEvent(vehicle, data)
	if g_server ~= nil then
		g_server:broadcastEvent(RekordiaEvent:new(vehicle, data), nil, nil, vehicle);
	else
		g_client:getServerConnection():sendEvent(RekordiaEvent:new(vehicle, data));
	end;
end;

SprayerFillActivatable = {}
local SprayerFillActivatable_mt = Class(SprayerFillActivatable);

function SprayerFillActivatable:new(sprayer)
    local self = {};
    setmetatable(self, SprayerFillActivatable_mt);

    self.sprayer = sprayer;
    self.activateText = "unknown";

    self.currentTrigger = nil;

    return self;
end;


function SprayerFillActivatable:getIsActivatable()
    self.currentTrigger = nil;
    if not self.sprayer:getIsActiveForInput() or self.sprayer.fillLevel == self.sprayer.capacity then
        return false;
    end;
    -- find the first trigger which is activable
    for i=1, table.getn(self.sprayer.sprayerFillTriggers) do
        local trigger = self.sprayer.sprayerFillTriggers[i];
        if trigger:getIsActivatable(self.sprayer) then
            self.currentTrigger = trigger;
            self:updateActivateText();
            return true;
        end;
    end;
    return false;
end;

function SprayerFillActivatable:onActivateObject()
    self.sprayer:setIsSprayerFilling(not self.sprayer.isSprayerFilling, self.currentTrigger.fillType, self.currentTrigger.isSiloTrigger);
    self:updateActivateText();
    g_currentMission:addActivatableObject(self);
end;

function SprayerFillActivatable:drawActivate()
    -- TODO draw icon
end;

function SprayerFillActivatable:updateActivateText()
    if self.sprayer.isSprayerFilling then
        self.activateText = string.format(g_i18n:getText("stop_refill_OBJECT"), self.sprayer.typeDesc);
    else
        self.activateText = string.format(g_i18n:getText("refill_OBJECT"), self.sprayer.typeDesc);
    end;
end; 